#include <avr/io.h>
#include <string.h>
#include <stdlib.h>

// include avrlib basics
#include "global.h"
#include "uart.h"
#include "rprintf.h"

// include network support
#include "net/net.h"
#include "net/nic.h"
#include "net/arp.h"
#include "net/icmp.h"
#include "net/ip.h"
#include "net/dhcp.h"
#include "net/netstack.h"

#include "ledmatrix.h"
#include "network.h"

// global variables
static volatile unsigned long UptimeMs;

// functions
void processCommand(u16 len, u08* data);
void serviceLocal(void);
void systickHandler(void);

// prototypes
int network_init(void)
{
  struct netEthAddr myEthAddress;
  
  // init network stack
  rprintf("Initializing Network Stack\r\n");
  netstackInit(IPADDRESS, NETMASK, GATEWAY);

  dhcpInit();
 
  nicGetMacAddress(&myEthAddress.addr[0]);
  rprintfProgStrM("Eth Addr is: "); 
  netPrintEthAddr(&myEthAddress);		
  rprintfCRLF();

  rprintfProgStrM("IP  Addr is: "); 
  netPrintIPAddr(ipGetConfig()->ip);	
  rprintfCRLF();
  
  rprintf("Network Stack is up!\r\n");
  
  rprintfProgStrM("\r\nInitializing Ethernet Controller\r\n");
  nicInit();
  
  rprintf("Starting packet receive loop\r\n");
  
  return 0;
}


void netstackUDPIPProcess(unsigned int len, udpip_hdr* packet)
{
  u16 payloadlen=0;
  u08* payloaddata=0;
  
  // get UDP payload length
  payloadlen = htons(packet->udp.udplen);
  payloadlen -= 8; // subtract header

  // get UDP payload data
  payloaddata = &((unsigned char*)packet)[IP_HEADER_LEN+UDP_HEADER_LEN];
  
//  rprintf("UDP packet, port: %d, len: %d\r\n", HTONS(packet->udp.destport), len);
//	debugPrintHexTable(len, (unsigned char*)packet);
  
  if(packet->udp.destport == HTONS(DHCP_UDP_CLIENT_PORT)) 
  {
    dhcpIn(payloadlen, (struct netDhcpHeader*)payloaddata);
  }
  else if(packet->udp.destport == HTONS(DSP_PORT)) 
  {
    switch(payloaddata[0]) {
      case DSP_CMD_IMAGE:
	if(payloadlen == DSP_MEM+1) {
	  ledmatrix_put(payloaddata+1);

	  // send ack for image command
	  payloaddata[0] = DSP_CMD_ACK;
	  payloaddata[1] = DSP_CMD_IMAGE;
	  u32 dst = htonl(packet->ip.srcipaddr);
	  udpSend(dst, DSP_PORT, 2, payloaddata);
	} else
	  rprintf("payload len %d does not match display "
		  "size %d\n\r", payloadlen, DSP_MEM);
	break;

      case DSP_CMD_BEEP:
	ledmatrix_beep();
	break;

      default:
	rprintf("Unknown command byte %d\n\r", payloaddata[0]);
	break;
    }
  }
}


void netstackTCPIPProcess(unsigned int len, tcpip_hdr* packet)
{
  rprintf("Received TCP/IP Packet: len=%d\r\n", len);
}

void serviceLocal(void) {
  static unsigned short last_sec = 0;

  // handle dhcp once a second
  if(seconds != last_sec) {
    last_sec = seconds;
    
    dhcpTimer();      
  }
}
